@@ -304,7 +304,6 @@ static gboolean
 volume_is_blank (GnomeVFSVolume *volume)
 {
 	LibHalContext *ctx;
-	DBusError      error;
 	char          *udi;
 	gboolean       is_blank;
 
@@ -313,12 +312,16 @@ volume_is_blank (GnomeVFSVolume *volume)
 		return FALSE;
 	}
 
+	is_blank = FALSE;
+
 	udi = gnome_vfs_volume_get_hal_udi (volume);
-	dbus_error_init (&error);
-	is_blank = libhal_device_get_property_bool (ctx,
-						    udi,
-						    "volume.disc.is_blank",
-						    NULL);
+	if (udi != NULL) {
+		is_blank = libhal_device_get_property_bool (ctx,
+							    udi,
+							    "volume.disc.is_blank",
+							    NULL);
+	}
+
 	g_free (udi);
 
 	return is_blank;
