@@ -418,12 +418,19 @@ fill_write_speeds (NautilusBurnDrive *drive)
 {
 	int max_speed;
 	int i;
+	int cdr_speed = 150;
+	int n_speeds;
 
 	max_speed = drive->priv->max_speed_write;
-	drive->priv->write_speeds = g_new0 (int, max_speed + 1);
+	n_speeds = max_speed / cdr_speed + 1;
 
-	for (i = 0; i < max_speed; i++) {
-		drive->priv->write_speeds[i] = max_speed - i;
+	drive->priv->write_speeds = g_new0 (int, n_speeds);
+
+	/* there is no point adding every kps to the list
+	   so we'll just show every 150 kbps */
+	for (i = 0; i < n_speeds; i++) {
+		/* in descending order */
+		drive->priv->write_speeds[n_speeds - 1 - i] = cdr_speed * i;
 	}
 }
 
@@ -604,7 +611,14 @@ nautilus_burn_drive_monitor_get_drive_for_device (NautilusBurnDriveMonitor *moni
 	for (l = monitor->priv->drives; l != NULL; l = l->next) {
 		drive = l->data;
 
-		if (drive->priv->device != NULL && strcmp (drive->priv->device, target) == 0) {
+		/*
+		 * On Solaris, the device name in the list of drives
+		 * is not expanded to the symlink so compare against
+		 * either the symlink value or the real device name.
+		 */
+		if (drive->priv->device != NULL &&
+		   (strcmp (drive->priv->device, target) == 0 ||
+                    strcmp (drive->priv->device, device) == 0)) {
 			ret = nautilus_burn_drive_ref (drive);
 			break;
 		}
@@ -624,13 +638,15 @@ hal_parse_write_speeds (char **strlist)
 	int   fields = 1;
 	int   i;
 
-	if (strlist == NULL)
+	if (strlist == NULL) {
 		return NULL;
+	}
 
 	fields = libhal_string_array_length (strlist);
 
-	if (fields == 0)
+	if (fields == 0) {
 		return NULL;
+	}
 
 	write_speeds = g_new0 (int, fields + 1);
 
@@ -666,6 +682,7 @@ hal_drive_from_udi (LibHalContext *ctx,
 	DBusError                 error;
 	NautilusBurnDrive        *drive;
 	char                    **write_speeds = NULL;
+	char                     *raw_device = NULL;
 
 	LIBHAL_CHECK_LIBHALCONTEXT (ctx, FALSE);
 
@@ -693,6 +710,7 @@ hal_drive_from_udi (LibHalContext *ctx,
 		LIBHAL_PROP_EXTRACT_BEGIN;
 
 		LIBHAL_PROP_EXTRACT_STRING  ("block.device",               drive->priv->device);
+		LIBHAL_PROP_EXTRACT_STRING  ("block.solaris.raw_device",   raw_device);
 		LIBHAL_PROP_EXTRACT_STRING  ("storage.model",              drive->priv->display_name);
 
 		LIBHAL_PROP_EXTRACT_INT ("storage.cdrom.read_speed",   drive->priv->max_speed_read);
@@ -713,6 +731,11 @@ hal_drive_from_udi (LibHalContext *ctx,
 		LIBHAL_PROP_EXTRACT_END;
 	}
 
+	if (raw_device != NULL) {
+		g_free (drive->priv->device);
+		drive->priv->device = raw_device;
+	}
+
 	drive->priv->drive_udi = g_strdup (udi);
 	drive->priv->write_speeds = hal_parse_write_speeds (write_speeds);
 	/* we do not own write_speeds so do not free it. */
@@ -720,7 +743,7 @@ hal_drive_from_udi (LibHalContext *ctx,
 		fill_write_speeds (drive);
 	}
 
-	if (! drive->priv->display_name) {
+	if (drive->priv->display_name == NULL) {
 		drive->priv->display_name = g_strdup_printf ("Unnamed Drive (%s)", drive->priv->device);
 	}
 
@@ -947,7 +970,7 @@ monitor_hal_shutdown (NautilusBurnDriveMonitor *monitor)
 	}
 
 #if USE_PRIVATE_DBUS_CONNECTION
-	dbus_connection_disconnect (dbus_connection);
+	dbus_connection_close (dbus_connection);
 #endif
 
 	if (! libhal_ctx_free (monitor->priv->ctx)) {
